/**
 * @Descripttion: markdown-it自定义配置
 * @Date:
 * @LastEditTime:
 */
const md = require('markdown-it')() // 引入markdown-it

const fs = require('fs')
const path = require('path')
const join = path.join

// option处理规则函数
function optionPreprocess(MarkdownIt, source) {
  // 属性描述表
  MarkdownIt.renderer.rules.table_open = function () {
    return '<table class="table">'
  }
  // 对于代码块去除v - pre, 添加高亮样式;
  const defaultRender = md.renderer.rules.fence
  MarkdownIt.renderer.rules.fence = (tokens, idx, options, env, self) => {
    const token = tokens[idx]
    // 判断该 fence 是否在 :::demo 内
    const prevToken = tokens[idx - 1]
    const isInDemoContainer =
      prevToken &&
      prevToken.nesting === 1 &&
      prevToken.info.trim().match(/^demo\s*(.*)$/)
    if (token.info === 'html' && isInDemoContainer) {
      return `<template slot="highlight"><pre v-pre><code class="html">${md.utils.escapeHtml(
        token.content
      )}</code></pre></template>`
    }
    return defaultRender(tokens, idx, options, env, self)
  }
  return source
}

/**
 * example：
 * :::demo
 * 代码内容
 * :::
 * Descripttion： option验证规则函数
 * 匹配:::后面的内容 nesting == 1,说明:::demo 后面有内容
 * m为数组,m[1]表示 代码内容
 */
function optionValidate(params) {
  return params.trim().match(/^demo\s*(.*)$/)
}

// option的render函数
function optionRender(tokens, idx, md) {
  const m = tokens[idx].info.trim().match(/^demo\s*(.*)$/)
  if (tokens[idx].nesting === 1) {
    //
    const description = m && m.length > 1 ? m[1] : '' // 获取正则捕获组中的描述内容,即::: demo xxx中的xxx
    const content =
      tokens[idx + 1].type === 'fence' ? tokens[idx + 1].content : ''

    return `<demo-block>
     <div slot="source">${content}</div>
     ${description ? `<div>${md.render(description)}</div>` : ''}
     `
  }
  return '</demo-block>'
}

/**
 * @desc 拼接文件链接
 * @param {*} __dirname 表示当前路径的绝对路径
 */
const resolve = dir => path.join(__dirname, '../', dir)

/**
 * @desc 获取组件入口
 * @param {*} path
 */
function getComponentEntries(path) {
  // 指定目录下所有文件名称
  let files = fs.readdirSync(resolve(path))
  const componentEntries = files.reduce((fileObj, item) => {
    //  文件路径
    const itemPath = join(path, item)
    //  在文件夹中-判断文件路径是否存在（statSync返回文件路径、isDirectory判断是否为目录）
    const isDir = fs.statSync(itemPath).isDirectory()
    const [name, suffix] = item.split('.')
    //  文件中的入口文件
    if (isDir) {
      fileObj[item] = resolve(join(itemPath, 'index.js'))
    }
    //  文件夹外的入口文件
    else if (suffix === 'js') {
      fileObj[name] = resolve(`${itemPath}`)
    }
    return fileObj
  }, {})

  return componentEntries
}

module.exports = {
  optionPreprocess,
  optionValidate,
  optionRender,
  getComponentEntries,
  resolve
}
